# Copyright 2023 The JAXampler Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .beta import Beta
from .boltzmann import Boltzmann
from .cauchy import Cauchy
from .chi2 import Chi2
from .crvs import ContinuousRV
from .exponential import Exponential
from .gamma import Gamma
from .logistic import Logistic
from .lognormal import LogNormal
from .normal import Normal
from .pareto import Pareto
from .rayleigh import Rayleigh
from .studentt import StudentT
from .triangular import Triangular
from .truncnormal import TruncNormal
from .truncpowerlaw import TruncPowerLaw
from .uniform import Uniform
from .weibull import Weibull
